"""
CLI interface
"""
import sys

from untruncate_json.untrunc import complete


def main() -> None:
    """Read string piped in and pass to complete function and then write result to console."""
    print(complete(sys.stdin.read()))



"""
Credit:
https://github.com/dphilipson/untruncate-json/blob/master/src/index.ts
License: MIT
"""
import json

import pytest

from untruncate_json import complete


def expect_unchanged(json_str):
    assert complete(json_str) == json_str


def test_returns_unmodified_valid_string():
    expect_unchanged('"Hello"')


def test_returns_unmodified_valid_string_with_bracket_characters():
    expect_unchanged('"}{]["')


def test_returns_unmodified_valid_string_with_escaped_quotes():
    expect_unchanged('"\\"Dr.\\" Leo Spaceman"')


def test_returns_unmodified_valid_string_with_unicode_escapes():
    expect_unchanged("ab\\u0065cd")


def test_returns_unmodified_valid_number():
    expect_unchanged("20")


def test_returns_unmodified_valid_boolean():
    expect_unchanged("true")
    expect_unchanged("false")


def test_returns_unmodified_valid_null():
    expect_unchanged("null")


def test_returns_unmodified_valid_array():
    expect_unchanged("[]")
    expect_unchanged('["a", "b", "c"]')
    expect_unchanged("[ 1, 2, 3 ]")


def test_returns_unmodified_valid_object():
    expect_unchanged("{}")
    expect_unchanged('{"foo": "bar"}')
    expect_unchanged('{ "foo": 2 }')


def test_returns_unmodified_compound_object():
    expect_unchanged(
        '{"s": "Hello", "num": 10, "b": true, "nul": "null", "o": { "s": "Hello2", "num": 11 }, "a": ["Hello", 10, { "s": "Hello3" }]}'
    )


def test_adds_missing_close_quote():
    assert complete('"Hello') == '"Hello"'


def test_cut_off_trailing_backslash():
    assert complete('"Hello\\') == '"Hello"'


def test_cut_off_partial_unicode_escape():
    assert complete('"ab\\u006') == '"ab"'


def test_adds_zero_to_number_cut_off_at_a_negative_sign():
    assert complete("-") == "-0"


def test_adds_zero_to_number_cut_off_at_a_decimal_point():
    assert complete("12.") == "12.0"


def test_adds_zero_to_number_cut_off_at_an_exponent():
    assert complete("12e") == "12e0"
    assert complete("12E") == "12E0"


def test_add_zero_to_number_cut_off_after_e():
    assert complete("12e+") == "12e+0"
    assert complete("12E-") == "12E-0"


def test_completes_boolean_and_null_literals():
    assert complete("tr") == "true"
    assert complete("fal") == "false"
    assert complete("nu") == "null"


def test_close_empty_array():
    assert complete("[") == "[]"


def test_close_array_with_items():
    assert complete('["a", "b"') == '["a", "b"]'


def test_close_a_list_ending_in_a_number():
    assert complete("[1, 2") == "[1, 2]"


def test_completes_boolean_and_null_literals_at_the_end_of_a_list():
    assert complete("[tr") == "[true]"
    assert complete("[true, fa") == "[true, false]"
    assert complete("[nul") == "[null]"


def test_removes_a_trailing_comma_to_end_a_list():
    assert complete("[1, 2,") == "[1, 2]"


def test_closes_an_empty_object():
    assert complete("{") == "{}"


def test_closes_an_object_after_key_value_pairs():
    assert complete('{"a": "b"') == '{"a": "b"}'
    assert complete('{"a": 1') == '{"a": 1}'


def test_custs_off_a_partial_key_in_an_object():
    assert complete('{"hel') == "{}"
    assert complete('{"hello": 1, "wo') == '{"hello": 1}'


def test_cuts_off_a_key_missing_a_colon_in_an_object():
    assert complete('{"hello') == "{}"
    assert complete('{"hello": 1, "world') == '{"hello": 1}'


def test_cuts_off_a_key_and_colon_without_a_value_in_an_object():
    assert complete('{"hello":') == "{}"
    assert complete('{"hello": 1, "world": ') == '{"hello": 1}'


def test_untruncates_a_value_in_an_object():
    assert complete('{"hello": "wo') == '{"hello": "wo"}'
    assert complete('{"hello": [1, 2') == '{"hello": [1, 2]}'


def test_handles_a_string_in_an_array_cut_off_at_a_backslash():
    assert complete('["hello\\') == '["hello"]'
    assert complete('["hello", "world\\') == '["hello", "world"]'


def test_handles_a_cut_off_string_in_an_array_with_an_escaped_character():
    assert complete('["hello", "\\"Dr.]\\" Leo Spaceman') == '["hello", "\\"Dr.]\\" Leo Spaceman"]'


def test_handles_a_string_in_an_object_key_cut_off_at_a_backslash():
    assert complete('{"hello\\') == "{}"
    assert complete('{"hello": 1, "world\\') == '{"hello": 1}'


def test_removes_cut_off_object_with_key_containing_escaped_characters():
    assert complete('{"hello\\nworld": ') == "{}"
    assert complete('{"hello": 1, "hello\\nworld') == '{"hello": 1}'


def test_should_produce_valid_json_wherever_truncation_occurs():
    json_str = """{
        "ab\\nc\\u0065d": ["ab\\nc\\u0065d", true, false, null, -12.3e-4],
        "": { "12": "ab\\nc\\u0065d"}
    }"""
    for i in range(1, len(json_str)):
        partial_json = json_str[:i]
        fixed_json = complete(partial_json)
        try:
            json.loads(fixed_json)
        except json.JSONDecodeError:
            pytest.fail(
                f"Failed to produce valid JSON.\n\nInput:\n\n{partial_json}\n\nOutput (invalid JSON):\n\n{fixed_json}\n"
            )



import json

from half_json.core import JSONFixer
from hypothesis import given, strategies
from truncjson import truncjson

from untruncate_json import complete


def test_array_one():
    assert complete("[1") == "[1]"


def test_array_two():
    assert complete("[1, 2") == "[1, 2]"


SAMPLE = json.dumps(
    {
        "name": "John",
        "age": 30,
        "city": "New York",
        "pets": [
            {"name": "Puffy", "type": "cat"},
            {"name": "Duffy", "type": "dog"},
        ],
    }
)


@given(strategies.integers(min_value=10, max_value=len(SAMPLE)))
def test_untruncate_various(length):
    # Can't assert because it sometimes returns valid falsy values.
    try:
        _ = json.loads(complete(SAMPLE[:length]))
    except json.JSONDecodeError:
        print("----")
        print(f"INPUT: {SAMPLE[:length]}")
        print(f"OUTPUT: {complete(SAMPLE[:length])}")
        raise


@given(strategies.integers(min_value=10, max_value=len(SAMPLE)))
def test_truncjson_various(length):
    try:
        json.loads(truncjson.complete(SAMPLE[:length]))
    except json.JSONDecodeError:
        print("----")
        print(f"INPUT: {SAMPLE[:length]}")
        print(f"OUTPUT: {truncjson.complete(SAMPLE[:length])}")
        raise


@given(strategies.integers(min_value=10, max_value=len(SAMPLE)))
def test_jsonfixer_various(length):
    try:
        json.loads(JSONFixer().fix(SAMPLE[:length]).line)
    except json.JSONDecodeError:
        print("----")
        print(f"INPUT: {SAMPLE[:length]}")
        print(f"OUTPUT: {JSONFixer().fix(SAMPLE[:length]).line}")
        raise


def test_string():
    assert complete('"Hello, Wor') == '"Hello, Wor"'


def test_object():
    assert complete('{"votes": [true, fa') == '{"votes": [true, false]}'


def test_number():
    assert complete("123.") == "123.0"



#!/usr/bin/env python

from __future__ import annotations

import glob
import os
import os.path
import sys
from typing import TYPE_CHECKING, Any


# we'll import stuff from the source tree, let's ensure is on the sys path
sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))

# This requires setuptools when building; setuptools is not needed
# when installing from a wheel file (though it is still needed for
# alternative forms of installing, as suggested by README.md).
from setuptools import Extension, find_packages, setup
# from setuptools.command.build_py import build_py

from untruncate_json.version import __version__ as version

if TYPE_CHECKING:
    from typing_extensions import TypeGuard

def is_list_of_setuptools_extension(items: list[Any]) -> TypeGuard[list[Extension]]:
    return all(isinstance(item, Extension) for item in items)


def find_package_data(base, globs, root="untruncate_json"):
    """Find all interesting data files, for setup(package_data=)

    Arguments:
      root:  The directory to search in.
      globs: A list of glob patterns to accept files.
    """

    rv_dirs = [root for root, dirs, files in os.walk(base)]
    rv = []
    for rv_dir in rv_dirs:
        files = []
        for pat in globs:
            files += glob.glob(os.path.join(rv_dir, pat))
        if not files:
            continue
        rv.extend([os.path.relpath(f, root) for f in files])
    return rv


package_data = ["py.typed"]

package_data += find_package_data(os.path.join("untruncated_json"), ["*.py", "*.pyi"])

everything = [os.path.join("untruncate_json", x) for x in find_package_data("untruncate_json", ["*.py"])]

from mypyc.build import mypycify

opt_level = os.getenv("MYPYC_OPT_LEVEL", "3")
debug_level = os.getenv("MYPYC_DEBUG_LEVEL", "1")
force_multifile = os.getenv("MYPYC_MULTI_FILE", "") == "1"
ext_modules = mypycify(
    everything,  # + ["--config-file=mypy_bootstrap.ini"],
    opt_level=opt_level,
    debug_level=debug_level,
    # Use multi-file compilation mode on windows because without it
    # our Appveyor builds run out of memory sometimes.
    multi_file=sys.platform == "win32" or force_multifile,
)
assert is_list_of_setuptools_extension(ext_modules), "Expected mypycify to use setuptools"

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Topic :: Software Development",
    "Typing :: Typed",
]

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="untruncate_json",
    version=version,
    description="Cleans up truncated json files.",
    long_description=long_description,
    author="Matthew Martin",
    author_email="matthewdeanmartin@gmail.com",
    url="https://github.com/matthewdeanmartin/untruncate_json/",
    license="MIT",
    py_modules=[],
    ext_modules=ext_modules,
    packages=find_packages(),
    package_data={"untruncate_json": package_data},
    entry_points={
        "console_scripts": [
            "untruncate_json=untruncate_json.__main__:main",
        ]
    },
    classifiers=classifiers,
    install_requires=[
        # nothing!
    ],
    python_requires=">=3.8",
    include_package_data=True,
    project_urls={
        "Documentation": "https://github.com/matthewdeanmartin/untruncate_json",
        "Repository": "https://github.com/matthewdeanmartin/untruncate_json",
    },
)



"""
Untruncates json. Machine translated from untruncate-json by dphilipson
"""
from __future__ import annotations
from enum import Enum
from typing import Optional


class ContextType(Enum):
    """The type of context that the parser is in."""

    TOP_LEVEL = "topLevel"
    STRING = "string"
    STRING_ESCAPED = "stringEscaped"
    STRING_UNICODE = "stringUnicode"
    NUMBER = "number"
    NUMBER_NEEDS_DIGIT = "numberNeedsDigit"
    NUMBER_NEEDS_EXPONENT = "numberNeedsExponent"
    TRUE = "true"
    FALSE = "false"
    NULL = "null"
    ARRAY_NEEDS_VALUE = "arrayNeedsValue"
    ARRAY_NEEDS_COMMA = "arrayNeedsComma"
    OBJECT_NEEDS_KEY = "objectNeedsKey"
    OBJECT_NEEDS_COLON = "objectNeedsColon"
    OBJECT_NEEDS_VALUE = "objectNeedsValue"
    OBJECT_NEEDS_COMMA = "objectNeedsComma"


class RespawnReason(Enum):
    """ "The reason why the parser needs to respawn."""

    STRING_ESCAPE = "stringEscape"
    COLLECTION_ITEM = "collectionItem"


def is_whitespace(char: str) -> bool:
    """Returns true if the character is whitespace."""
    return char in "\u0020\u000D\u000A\u0009"


def complete(json_str: str) -> str:
    """Completes the json string."""
    context_stack: list[ContextType] = [ContextType.TOP_LEVEL]
    position = 0
    respawn_position: Optional[int] = None
    respawn_stack_length: Optional[int] = None
    respawn_reason: Optional[RespawnReason] = None

    def push(context: ContextType) -> None:
        """Pushes the context onto the stack."""
        context_stack.append(context)

    def replace(context: ContextType) -> None:
        """Replaces the context on the stack."""
        context_stack[-1] = context

    def set_respawn(reason: RespawnReason) -> None:
        """Sets the respawn position."""
        nonlocal respawn_position, respawn_stack_length, respawn_reason
        if respawn_position is None:
            respawn_position = position
            respawn_stack_length = len(context_stack)
            respawn_reason = reason

    def clear_respawn(reason: RespawnReason) -> None:
        """Clears the respawn position."""
        nonlocal respawn_position, respawn_stack_length, respawn_reason
        if reason == respawn_reason:
            respawn_position = None
            respawn_stack_length = None
            respawn_reason = None

    def pop() -> None:
        """Pops the context stack."""
        context_stack.pop()

    def dont_consume_character() -> None:
        """Doesn't consume the character."""
        nonlocal position
        position -= 1

    def start_any(char: str) -> None:
        """Starts any context."""
        if "0" <= char <= "9":
            push(ContextType.NUMBER)
            return
        if char == '"':
            push(ContextType.STRING)
            return
        if char == "-":
            push(ContextType.NUMBER_NEEDS_DIGIT)
            return
        if char == "t":
            push(ContextType.TRUE)
            return
        if char == "f":
            push(ContextType.FALSE)
            return
        if char == "n":
            push(ContextType.NULL)
            return
        if char == "[":
            push(ContextType.ARRAY_NEEDS_VALUE)
            return
        if char == "{":
            push(ContextType.OBJECT_NEEDS_KEY)
            return

    while position < len(json_str):
        char = json_str[position]
        current_context = context_stack[-1]

        if current_context == ContextType.TOP_LEVEL:
            start_any(char)
        elif current_context == ContextType.STRING:
            if char == '"':
                pop()
            elif char == "\\":
                set_respawn(RespawnReason.STRING_ESCAPE)
                push(ContextType.STRING_ESCAPED)
        elif current_context == ContextType.STRING_ESCAPED:
            if char == "u":
                push(ContextType.STRING_UNICODE)
            else:
                clear_respawn(RespawnReason.STRING_ESCAPE)
                pop()
        elif current_context == ContextType.STRING_UNICODE:
            if position - json_str.rfind("u", 0, position) == 4:
                clear_respawn(RespawnReason.STRING_ESCAPE)
                pop()
        elif current_context == ContextType.NUMBER:
            if char == ".":
                replace(ContextType.NUMBER_NEEDS_DIGIT)
            elif char in ["e", "E"]:
                replace(ContextType.NUMBER_NEEDS_EXPONENT)
            elif not "0" <= char <= "9":
                dont_consume_character()
                pop()
        elif current_context == ContextType.NUMBER_NEEDS_DIGIT:
            replace(ContextType.NUMBER)
        elif current_context == ContextType.NUMBER_NEEDS_EXPONENT:
            if char in ["+", "-"]:
                replace(ContextType.NUMBER_NEEDS_DIGIT)
            else:
                replace(ContextType.NUMBER)
        elif current_context in [ContextType.TRUE, ContextType.FALSE, ContextType.NULL]:
            if not "a" <= char <= "z":
                dont_consume_character()
                pop()
        elif current_context == ContextType.ARRAY_NEEDS_VALUE:
            if char == "]":
                pop()
            elif not is_whitespace(char):
                clear_respawn(RespawnReason.COLLECTION_ITEM)
                replace(ContextType.ARRAY_NEEDS_COMMA)
                start_any(char)
        elif current_context == ContextType.ARRAY_NEEDS_COMMA:
            if char == "]":
                pop()
            elif char == ",":
                set_respawn(RespawnReason.COLLECTION_ITEM)
                replace(ContextType.ARRAY_NEEDS_VALUE)
        elif current_context == ContextType.OBJECT_NEEDS_KEY:
            if char == "}":
                pop()
            elif char == '"':
                set_respawn(RespawnReason.COLLECTION_ITEM)
                replace(ContextType.OBJECT_NEEDS_COLON)
                push(ContextType.STRING)
        elif current_context == ContextType.OBJECT_NEEDS_COLON:
            if char == ":":
                replace(ContextType.OBJECT_NEEDS_VALUE)
        elif current_context == ContextType.OBJECT_NEEDS_VALUE:
            if not is_whitespace(char):
                clear_respawn(RespawnReason.COLLECTION_ITEM)
                replace(ContextType.OBJECT_NEEDS_COMMA)
                start_any(char)
        elif current_context == ContextType.OBJECT_NEEDS_COMMA:
            if char == "}":
                pop()
            elif char == ",":
                set_respawn(RespawnReason.COLLECTION_ITEM)
                replace(ContextType.OBJECT_NEEDS_KEY)
        position += 1

    if respawn_stack_length is not None:
        context_stack = context_stack[:respawn_stack_length]

    # Final assembly of the result
    result = [json_str[:respawn_position]] if respawn_position is not None else [json_str]

    def finish_word(word: str) -> None:
        """Finishes the word."""
        result.append(word[len(json_str) - json_str.rfind(word[0]) :])

    for i in range(len(context_stack) - 1, -1, -1):
        context = context_stack[i]

        if context == ContextType.STRING:
            result.append('"')
        elif context in (ContextType.NUMBER_NEEDS_DIGIT, ContextType.NUMBER_NEEDS_EXPONENT):
            result.append("0")
        elif context == ContextType.TRUE:
            finish_word("true")
        elif context == ContextType.FALSE:
            finish_word("false")
        elif context == ContextType.NULL:
            finish_word("null")
        elif context in (ContextType.ARRAY_NEEDS_VALUE, ContextType.ARRAY_NEEDS_COMMA):
            result.append("]")
        elif context in (
            ContextType.OBJECT_NEEDS_KEY,
            ContextType.OBJECT_NEEDS_COLON,
            ContextType.OBJECT_NEEDS_VALUE,
            ContextType.OBJECT_NEEDS_COMMA,
        ):
            result.append("}")

    return "".join(result)
